// ====================================================
//
// Copyright (c) 2002 Sean Wilson. All Rights Reserved.
//
// ====================================================

#ifndef HEADER_SEANW_EXCEPTION
#define HEADER_SEANW_EXCEPTION

#include "msvc_warnings_off.h"
#include <stdexcept>

#ifdef _WIN32
#include <windows.h>
#include <new.h>
#include <eh.h>
#endif

#include "msvc_warnings_on.h"

namespace seanw
{
namespace exception
{

/** Win32 specific. Win32 exceptions (C structured exceptions) wrapper. */
#ifdef _WIN32
class win32Exception : public std::exception
{
public:
    /** Constructs a Win32 exception.
	 *  @param exceptionCode Win32 exception code.
	 */
	explicit win32Exception(unsigned int exceptionCode);

	virtual const char* what() const
	{
		return message;
	}
	
	virtual ~win32Exception() { }

private:
    const char* message;
};
#endif

/** Win32 specific. Win32 exceptions (C structured exceptions) handler. Simple rethrows exception as a C++ exception.
 *  @throw win32Exception
 */
#ifdef _WIN32
void softwareExceptionTranslator(unsigned int exceptionCode, EXCEPTION_POINTERS* /*exceptionRecord*/);
#endif

/** Out of memory handler which immediately throws a std::bad_alloc exception. */
#ifdef _WIN32
int badAllocHandler(::size_t /*allocationSize*/);
#endif

}
}

#endif