// ====================================================
//
// Copyright (c) 2002 Sean Wilson. All Rights Reserved.
//
// ====================================================

#ifndef HEADER_SEANW_GRAPHICS
#define HEADER_SEANW_GRAPHICS

#include "color.h"

#ifdef _WIN32
#include "msvc_warnings_off.h"
#include <windows.h>
#include "msvc_warnings_on.h"
#endif

namespace seanw
{
namespace graphics
{

/** Used to render onto a graphics context. */
class Graphics
{
public:
	/** Win32 specific. Contructs new graphics context.
	 *  @param initDC device context rendering will take place in.
	 */
	#ifdef _WIN32
	explicit Graphics(::HDC initDc);
	#endif

	~Graphics();

	/** Sets current rendering color.
	 *  @param color new rendering color.
	 */
	void setColor(const Color& color);

	/** Draws a filled circle using current color.
	 *  @param x      x co-ordinate of circle center.
	 *  @param y      y co-ordinate circle center.
	 *  @param radius circle radius.
	 */
	void fillCircle(int x, int y, int radius);

	/** Draws an circle outline using current color.
	 *  @param x      x co-ordinate of circle center.
	 *  @param y      y co-ordinate circle center.
	 *  @param radius circle radius.
	 */
	void drawCircle(int x, int y, int radius);
	
	/** Draws a straight line using current color.
	 *  @param color  line color.
	 *  @param x1     x co-ordinate of line start.
	 *  @param y1     y co-ordinate of line start.
	 *  @param x2     x co-ordinate of line end.
	 *  @param y2     y co-ordinate of line end.
	 */
	void drawLine(int x1, int y1, int x2, int y2);

	/** Draws a filled rectangle using current color.
	 *  @param x      top left x co-ordinate.
	 *  @param y      top left y co-ordinate.
	 *  @param width  rectangle width.
	 *  @param height rectangle height.
	 */
	void fillRectangle(int x, int y, int width, int height);

private:
	// Prevent copy construction
	Graphics(const Graphics&);
	// Prevent assignment
	const Graphics& operator=(const Graphics&);

	#ifdef _WIN32
	/** Win32 specific. Device context to draw to. */
	::HDC dc;
	
	/** Win32 specific. Currently selected brush object in DC. */
	::HBRUSH currentBrush;
	/** Win32 specific. Previously selected brush object in DC. */
	::HBRUSH lastBrush;
	
	/** Win32 specific. Currently selected pen object in DC. */
	::HPEN currentPen;
	/** Win32 specific. Previously selected pen object in DC. */
	::HPEN lastPen;
	#endif
};

}
}

#endif