#include "timer.h"

namespace seanw
{
namespace timer
{

#ifdef _WIN32
const long Timer::FREQUENCY_TIME_GET_TIME       = 1000;
const long Timer::FREQUENCY_PERFORMANCE_COUNTER = Timer::getTimerFrequency();
#endif

Timer::Timer()
{
	reset();
}

#ifdef _WIN32
long Timer::getTimerFrequency()
{
	::LARGE_INTEGER performanceCounterFrequency;
	
	if (::QueryPerformanceFrequency(static_cast<::LARGE_INTEGER*>(&performanceCounterFrequency)))
	{
		return performanceCounterFrequency.LowPart;
	}

	return 0;
}
#endif

#ifdef _WIN32
double Timer::getTime()
{
	// Return less accurate timer time if performance counter not available
	if (FREQUENCY_PERFORMANCE_COUNTER == 0)
	{
		return static_cast<double>(::timeGetTime()) / FREQUENCY_TIME_GET_TIME;
	}

	::LARGE_INTEGER ticks;
	::QueryPerformanceCounter(static_cast<::LARGE_INTEGER*>(&ticks));

	return static_cast<double>(ticks.QuadPart) / FREQUENCY_PERFORMANCE_COUNTER;
}
#endif

void Timer::reset()
{
	startTime = getTime();
}

double Timer::getTimeElapsed() const
{
	return getTime() - startTime;
}

void Timer::waitUntilTimeElapsed(double waitLength) const
{
	while (getTimeElapsed() < waitLength)
	{
	}
}

void Timer::wait(double waitLength)
{
	Timer().waitUntilTimeElapsed(waitLength);
}

}
}