// ====================================================
//
// Copyright (c) 2002 Sean Wilson. All Rights Reserved.
//
// ====================================================

#ifndef HEADER_SEANW_TIMER
#define HEADER_SEANW_TIMER

#ifdef _WIN32
#include "msvc_warnings_off.h"
#include <windows.h>
#include <mmsystem.h>
#include "msvc_warnings_on.h"
#endif

namespace seanw
{
namespace timer
{

/** Timer which uses highest precision timing device available. */
class Timer
{
public:
	/** Contructs a new timer with start time set to creation time. */
	Timer();
	
	/** Sets start time to now. */
	void reset();

	/** Waits until a specific time period has elasped since timer start was set.
	 *  @param waitLength time to wait in seconds.
	 */
	void waitUntilTimeElapsed(double waitLength) const;

	/** Returns seconds elapsed since timer start was set. */
	double getTimeElapsed() const;

	/** Waits until a specific time period has elapsed.
	 *  @param waitLength time to wait in seconds.
	 */
	static void wait(double waitLength);

	/** Returns current number of seconds counted. This is only for relative measurements. */
	static double getTime();

private:
	#ifdef _WIN32
	/** Win32 specific. Returns ticks per second frequency for hardware high-resolution performance counter. Returns 0 if one does not exists. */
	static long getTimerFrequency();
	#endif

	#ifdef _WIN32
	/** Win32 specific. Ticks per second frequency of ::timeGetTime() function. */
	static const long FREQUENCY_TIME_GET_TIME;
	/** Win32 specific. Ticks per second frequency of hardware high-resolution performance counter. */
	static const long FREQUENCY_PERFORMANCE_COUNTER;
	#endif

	/** Start time that is used to compare current time to. */
	double startTime;
};

}
}

#endif